﻿using System;
using System.Windows.Forms;

// Implements the logic of the game
// The displaying of the game is the responsibility of a separate class.
//
//
//
// Ian Smith                     26 Nov 2020       Original Version
//--------------------------------------------------------------------------------------------------------------------------

namespace TestGame.Game
{
    class GameLogic
    {
        // Public properties
        // Grid size and are all the tiles set
        public Int16 GridSizeX { get { return SizeX; } }
        public Int16 GridSizeY { get { return SizeY; } }
        public Boolean IsGameOver { get { return AllTilesOff; } }

        // Private variables
        private Int16 SizeX;            // Grid width
        private Int16 SizeY;            // Gird height
        private Boolean[,] TileStatus;  // On/Off Status of tile
        private Boolean AllTilesOff;     // Are all the tiles on

        private const Int32 TilesOnAtStart = 4;     // max Number of tiles to set on at the start

        // The constructor is called with the size of the desired grid
        public GameLogic(Int16 GridSizeX, Int16 GridSizeY)
        {   // Save the size of grid requested
            SizeX = GridSizeX;
            SizeY = GridSizeY;

            // restrict the grid to 1 to 64
            if ((GridSizeX < 1) || (GridSizeX > 64)) throw new Exception("Failed to initialise GameLogic as GridSizeX is out of range");
            if ((GridSizeY < 1) || (GridSizeY > 64)) throw new Exception("Failed to initialise GameLogic as GridSizeX is out of range");

            // Set the game to a new game
            ResetGame();
        }

        public void ResetGame()
        {   // resets the game to a new game of the same size

            // Initialise the tile status array, this is a bit of a waste in reset mode as we could just clear the existing array
            TileStatus = new Boolean[GridSizeX, GridSizeY];

            // Not all tiles are on
            AllTilesOff = false;

            // Set some tiles on
            SetSomeLightsOn();
        }


        public Boolean GetTileStatus(Int16 TileX, Int16 TileY)
        {   // Get the on/off status of one tile
            // This function does not trap any exceptions insteal it allows the exception to bubble up
            return TileStatus[TileX, TileY];
        }

        private void SetSomeLightsOn()
        {   // Sets some random tiles on
            // This function does not trap any exceptions insteal it allows the exception to bubble up
            Int32 TileX, TileY;
            Random Rand = new Random();

            // Loop for the specified number of lights to set
            for (Int16 lcTiles = 1; lcTiles <= TilesOnAtStart; lcTiles++)
            {
                // Get random position of tile
                TileX = Rand.Next(GridSizeX);
                TileY = Rand.Next(GridSizeY);

                // Set status to true;
                TileStatus[TileX, TileY] = true;

                // we don't check that the tile for being on, so we could get less than TilesOnAtStart on but there must always be at least one.
            }
        }

        public Boolean ToggleTile(Int16 TileX, Int16 TileY, Boolean CheckOthers)
        {   // Get the on/off status of one tile
            Boolean boolRes = false;
            Boolean TempAllTilesOff;

            try
            {
                // Do we need to skip this tile as it out of range
                if ((TileX >= 0) && (TileY >= 0) && (TileX < SizeX) && (TileY < SizeY))
                {   // No so toggle tile
                    TileStatus[TileX, TileY] = !TileStatus[TileX, TileY];
                }

                // Do we need to check companions
                if (CheckOthers == true)
                {   // Toggle the four companions
                    if (ToggleTile((Int16)(TileX - 1), TileY, false) == false) return false;
                    if (ToggleTile((Int16)(TileX + 1), TileY, false) == false) return false;
                    if (ToggleTile(TileX, (Int16)(TileY - 1), false) == false) return false;
                    if (ToggleTile(TileX, (Int16)(TileY + 1), false) == false) return false;

                    // As this is a bit of a rush, do a simple and safe test for all tiles on
                    TempAllTilesOff = true;

                    // Loop for all of the tiles
                    for (Int16 lcY = 0; lcY < GridSizeY; lcY++)
                    {
                        for (Int16 lcX = 0; lcX < GridSizeX; lcX++)
                        {   // is this one tile on
                            if (TileStatus[lcX, lcY] == true) { TempAllTilesOff = false; break; }
                        }

                        // Can we finish?
                        if (TempAllTilesOff == false) break;
                    }

                    // Make the results available as all were checked
                    AllTilesOff = TempAllTilesOff;
                }

                // Set function return variable as a success
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }
        //
        // ------------------------------------------------------------------------------------------------------------------
        // Test functions
        // ------------------------------------------------------------------------------------------------------------------
        //
        public Boolean TestFinishGame()
        {   // A test function that completes the game
            Boolean boolRes = false;

            try
            {
                // Loop for all of the tiles
                for (Int16 lcY = 0; lcY < GridSizeY; lcY++)
                {
                    for (Int16 lcX = 0; lcX < GridSizeX; lcX++)
                    {   // set tile on
                        TileStatus[lcX, lcY] = false;
                    }
                }

                // Set all tiles on flag
                AllTilesOff = true;

                // Set function return variable as a success
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }
    }
}
