﻿using System;
using System.Windows.Forms;
using System.Drawing;

// The logic associated with drawing the game area
//
//
//
// Ian Smith                     26 Nov 2020       Original Version
//-----------------------------------------------------------------------------------------------------------------------
namespace TestGame
{
    public partial class formMain : Form
    {
        // Size of the game grid
        private Int16 SizeX;
        private Int16 SizeY;

        // Dimensions of box and gap for drawing the game grid
        private Int32 BoxWidth, BoxHeight;
        private Int32 InterboxGapX, InterboxGapY;
        private Int32 BoxStartX, BoxStartY;

        private GroupBox GameArea;

        // Colours for game tiles
        private Color TileEmpty = SystemColors.Control;
        private Color TileOff = Color.MediumSeaGreen;
        private Color TileOn = Color.LightGreen;

        // base Name of the picture box that is a tile
        private const String TileBaseName = "picGameTile";
        private const String XYSeparator = "_";

        // Public properties
        // Grid size
        public Int16 GridSizeX { get { return SizeX; } }
        public Int16 GridSizeY { get { return SizeY; } }


        public void CreateGameDisplay(Int16 GridSizeX, Int16 GridSizeY, GroupBox GroupBox)
        {   // Save the size of grid requested
            SizeX = GridSizeX;
            SizeY = GridSizeY;
            GameArea = GroupBox;

            // restring the grid to 1 to 64
            if ((GridSizeX < 1) || (GridSizeX > 64)) throw new Exception("Failed to initialise GameDisplay as GridSizeX is out of range");
            if ((GridSizeY < 1) || (GridSizeY > 64)) throw new Exception("Failed to initialise GameDisplay as GridSizeX is out of range");

            // Calculate the size of the boxes to draw
            if (CalculateGridDimensions(GroupBox) == false) throw new Exception("Failed to initialise GameDisplay CalculateGridDimensions() failed");

            // draw an empty grid
            if (DrawEmptyGrid(GroupBox) == false) throw new Exception("Failed to initialise GameDisplay DrawEmptyGrid() failed");
        }

        public Boolean CalculateGridDimensions(GroupBox GroupBox)
        {   // This function calculates the size of the boxes
            Boolean boolRes = false;
            Double TotalWidth, TotalHeight;
            Double dblInterboxGapX, dblInterboxGapY;
            Double TotalInterboxGapX, TotalInterboxGapY;
            Int32 LabelHeight;
            try
            {
                // Allow space for the group box label
                LabelHeight = 20;

                // Get the total width and height of the binding area.
                TotalWidth = GroupBox.Width;
                TotalHeight = GroupBox.Height - LabelHeight;


                // Subtract a margin of 5% top, left, bottom, right
                TotalWidth *= 0.95;
                TotalHeight *= 0.95;
                BoxStartX = (Int32)(TotalWidth * 0.05);
                BoxStartY = (Int32)(TotalHeight * 0.05) + LabelHeight;

                // Calculate an interbox gap of 2%
                TotalInterboxGapX = TotalWidth * (0.02 * (SizeX - 1));
                TotalInterboxGapY = TotalHeight * (0.02 * (SizeY - 1));
                dblInterboxGapX = TotalInterboxGapX / (SizeX - 1);
                dblInterboxGapY = TotalInterboxGapY / (SizeY - 1);
                InterboxGapX = (Int32)dblInterboxGapX;
                InterboxGapY = (Int32)dblInterboxGapY;

                // Calculate box size
                BoxWidth = (Int32)((TotalWidth - TotalInterboxGapX) / SizeX);
                BoxHeight = (Int32)((TotalHeight - TotalInterboxGapY) / SizeY);

                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }

        public Boolean DrawEmptyGrid(GroupBox GroupBox)
        {   // Create controls within the game area to display a blank grid
            Boolean boolRes = false;
            Int32 DrawX, DrawY;
            PictureBox NewPictureBox;
            try
            {
                // Work out where to start adding the picuure controls
                DrawX = BoxStartX;
                DrawY = BoxStartY;

                // Loop for grids width and height
                for (Int16 lcY = 0; lcY < SizeY; lcY++)
                {
                    for (Int16 lcX =0; lcX < SizeX; lcX++)
                    {
                        // Create a new picture box with a name that describes its position and place it
                        NewPictureBox = new PictureBox();
                        NewPictureBox.Name = TileBaseName + lcX + lcY;
                        NewPictureBox.Tag = lcX + XYSeparator + lcY;
                        NewPictureBox.Width = BoxWidth;
                        NewPictureBox.Height = BoxHeight;
                        NewPictureBox.BackColor = TileEmpty;
                        NewPictureBox.Location = new System.Drawing.Point(DrawX, DrawY);
                        NewPictureBox.MouseClick += MouseTile_Click;
                        GroupBox.Controls.Add(NewPictureBox);

                        // Update position of box
                        DrawX += InterboxGapX + BoxWidth;
                    }

                    // Reset the X position and update the Y position
                    DrawX = BoxStartX;
                    DrawY += BoxHeight + InterboxGapY;

                }
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }

        public Boolean SetTileColour(GroupBox GameArea, Int16 GridX, Int16 GridY, Boolean TileSet)
        {   // Sets a title to on or off
            Boolean boolRes = false;
            String ControlName;

            try
            {
                // The control was named with its grid position so construct its name/
                ControlName = TileBaseName + GridX + GridY;

                // Set the color of the tile
                if (TileSet == true)
                {  // Tile is set
                    GameArea.Controls[ControlName].BackColor = TileOn;
                }
                else
                {  // Tile is not set
                    GameArea.Controls[ControlName].BackColor = TileOff;
                }
                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }

        public Boolean GetTileFromMouseClick(PictureBox sender, ref Int16 TileX,ref Int16 TileY)
        {   // gets the X and Y position from the tile name
            Boolean boolRes = false;
            Int32 Pos;

            try
            {
                // Get the x and y from the tag property

                // Firstly find the separator
                Pos = sender.Tag.ToString().IndexOf(XYSeparator); 
                if (Pos < 1)
                {   // Failed to find the x/y separator
                    MessageBox.Show("Failed to find the x/y separator in GetTileFromMouseClick()", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                // Then get the values
                TileX = Convert.ToInt16(sender.Tag.ToString().Substring(0, Pos));
                TileY = Convert.ToInt16(sender.Tag.ToString().Substring(Pos+1));

                boolRes = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return boolRes;
        }

    }
}
